@@ -1,6 +1,6 @@
 pub mod args;
 pub mod ask;
-pub mod dir_pair;
+pub mod dirs;
 pub mod dup;
 pub mod dup_report;
 pub mod file_pair;
@@ -11,7 +11,7 @@ pub mod removal_report;
 pub use {
     args::*,
     ask::*,
-    dir_pair::*,
+    dirs::*,
     dup::*,
     dup_report::*,
     file_pair::*,
